#!/usr/bin/perl

#┌─────────────────────────────────
#│ WEB PATIO : patio.cgi - 2022/03/26
#│ copyright (c) kentweb, 1997-2022
#│ https://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│ WebPatio v6.1 View Limit : patio.cgi - 2023/10/07
#│ Edited by TRANSFORM Since 2012
#│ https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
&error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# 認証モード
my %au = authent() if ($cf{authkey});

# 処理分岐
find_log() if ($in{find});
if ($in{read}) { read_log(); }
if ($in{edit}) { pwd_form(); }
if ($in{mode} eq 'form') { form_page(); }
if ($in{mode} eq 'expound') { expound_page(); }
if ($in{mode} eq 'find') { find_page(); }
if ($in{mode} eq 'note') { note_page(); }
if ($in{mode} eq 'past') { past_page(); }
if ($in{mode} eq 'album') { album_page(); }
bbs_list();

#-----------------------------------------------------------
#  メニュー部表示
#-----------------------------------------------------------
sub bbs_list {
	# アラーム数定義
	my $alarm = int ($cf{m_max} * 0.9);

	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';
	
	# キーワード配列化
	$in{q} =~ s/　/ /g;
	my @q = split(/\s+/,$in{q});

	# ページ数
	my $pg = $in{pg} || 0;

	# スレッド表示
	my ($i,@log);
	open(IN,"$cf{nowfile}") or &error("open err: $cf{nowfile}");
	my $top = <IN>;
	while (<IN>) {

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{pgmax_now});

		push(@log,$_);
	}
	close(IN);

	# 繰越ボタン作成
	my $page_btn = make_pgbtn($i,$pg,$cf{pgmax_now});

	# カウンタ
	my $counter = bbs_count() if ($cf{counter});

	# テンプレート読込
	open(IN,"$cf{tmpldir}/bbs.html") or &error("open err: bbs.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!new_mark!/$cf{new_mark}/g;
	$tmpl =~ s/!new_time!/$cf{new_time}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
	$tmpl =~ s/!counter!/$counter/g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
		? ($1,$2,$3) : error("テンプレート不正");

	foreach ($head,$foot) {

		# 認証モード
		if ($cf{authkey}) {
			s/!login-name!/$au{name}/g;
		} else {
			s/<!-- auth_begin -->.+?<!-- auth_end -->//sg;
		}

		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!homepage!/$cf{homepage}/g;
		s|!ico:(\w+\.\w+)!|<img src="$cf{iconurl}/$1">|g;
		s/!page-btn!/$page_btn/g;
		s/!alarm!/$alarm/g;
	}

	# 認証クッキー
	if ($cf{authkey} && $in{mode} eq 'login') {
		&set_cookie('CGISESSID',$au{sid});
	}

	# 画面表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@log) {
		chomp;
		my ($num,$sub,$res,$nam,$upd,$last,$key,$upl) = split(/<>/);
		my $ukey = $upl ? 1 : 0;

		# 参照カウンタ
		open(NO,"$cf{logdir}/$num.dat");
		my $data = <NO>;
		close(NO);
		my ($cnt) = (split(/:/,$data))[0];

		# データ読み込み
		open(DAT,"< $cf{logdir}/$num.cgi") or &error("open err: $cf{logdir}/$num.cgi");
		my $top = <DAT>;
		my $par = <DAT>;
			my ($no2,$date,$url,$tim,$tube) = (split(/<>/, $par))[0,5,8,11,16];

		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;

		# 所定時間以内の投稿は[NEWマーク]表示
		my $time = time;
		my $newsign;
		if ($time - $tim > $cf{new_time} * 3600) { $newsign = ""; }
		else { $newsign = $cf{new_mark}; }

		my $tmp = $loop;
		$tmp =~ s/!ico!/icon_img($key,$res,$alarm,$upl,$tube)/eg;
		$tmp =~ s|!topic!|<a href="$cf{bbs_cgi}?read=$num&ukey=$ukey">$sub</a>|g;
		$tmp =~ s/!name!/$nam/g;
		$tmp =~ s/!res!/$res/g;
		$tmp =~ s/!count!/$cnt/g;
		$tmp =~ s/!update!/$upd/g;
		$tmp =~ s/!last!/$last/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!url!/$url/g;
		$tmp =~ s/!new!/$newsign/g;

		print $tmp;
	}

	# フッター
	footer($foot);
}

#-----------------------------------------------------------
#  フォルダーアイコン
#-----------------------------------------------------------
sub icon_img {
	my ($key,$res,$alarm,$upl,$tube) = @_;

	# アイコン判断
	my $ico;
	if ($key eq '0') { $ico = 0; }
	elsif ($key == 2) { $ico = 2; }
	elsif ($key == 3) { $ico = 3; }
	elsif ($res >= $alarm) { $ico = 'alerm'; }
	elsif ($upl) { $ico = 'image'; }
	elsif ($tube) { $ico = 'tube'; }
	else { $ico = 1; }

	return qq|<img src="$cf{iconurl}/$cf{fld_icon}->{$ico}">|;
}

#-----------------------------------------------------------
#  記事閲覧
#-----------------------------------------------------------
sub read_log {
	$in{read} =~ s/\D//g;

	my $vcue;
	open(DAT,"$cf{nowfile}") or error("open err: $cf{nowfile}");
	my $top = <DAT>;
	while(<DAT>) {
		my ($num,$sub,$res,$nam,$upd,$last,$key,$upl) = split(/<>/);

		if ($in{read} == $num) {

			$vcue = $key;
			passwd2(%in) if ($key == 3);
		}
	}
	close(DAT);

	# クッキー取得
	my ($ck_nam,$ck_eml,$ck_url,$ck_mlo,$ck_col) = &get_cookie;
	$ck_url ||= 'http://';

	# 色選択ボタン
	my @col = split(/\s+/, $cf{colors});
	my $color;
	foreach (0 .. $#col) {
		if ($ck_col == $_) {
			$color .= qq|<input type="radio" name="color" value="$_" checked>|;
		} else {
			$color .= qq|<input type="radio" name="color" value="$_">|;
		}
		$color .= qq|<span style="color:$col[$_]">●</span>\n|;
	}

	# アイコン
	my $smile;
	my @smile = split(/\s+/, $cf{smile});
	foreach (0 .. $#smile) {
		$smile .= qq|<a href="javascript:face('{ico:$_}')"><img src="$cf{iconurl}/$smile[$_]"></a>|;
	}

	# アラーム数定義
	my $alarm = int ($cf{m_max} * 0.9);

	# ページ数定義
	my $pg = $in{pg} || 0;

	# スレッド読み込み
	my @log;
	open(IN,"$cf{logdir}/$in{read}.cgi") or &error("open err: $in{read}.cgi");
	my $top = <IN>;
	my $par = <IN>;
	my ($no,$sub,$res,$key) = split(/<>/, $top);

	my $i = 0;
	while(<IN>) {
		$i++;
		# 返信記事の表示順 (1=昇順)
		if ($cf{asc_order}) {
			next if ($i <= $pg);
			last if ($i > $cf{pg_max} + $pg);
			push(@log,$_);
		# 返信記事の表示順 (0=降順)
		} else {
			next if ($i <= $res - $cf{pg_max} - $pg);
			last if ($i > $res - $pg);
			unshift(@log,$_);
		}
	}
	close(IN);

	# 返信フォーム
	my $resfm = 1;
	if ($key != 1 || $in{log} eq 'past') {
		$resfm = 0;
	}

	# 繰越ボタン作成
	my $page_btn = make_pgbtn($res,$pg,$cf{pg_max},"read=$in{read}");

	# テンプレート読込
	open(IN,"$cf{tmpldir}/read.html") or &error("open err: read.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0 && $resfm) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt) = cap::make( $cf{captcha_key}, $cf{cap_len} );
	} else {
		$tmpl =~ s/<!-- captcha_begin -->.+<!-- captcha_end -->//s;
	}

	# email表示
	my $op_mlo;
	my @mlo = ('非表示','表示');
	foreach (1,0) {
		if ($ck_mlo eq $_) {
			$op_mlo .= qq|<option value="$_" selected>$mlo[$_]\n|;
		} else {
			$op_mlo .= qq|<option value="$_">$mlo[$_]\n|;
		}
	}

	# 親記事
	my ($no2,$sub,$nam,$eml,$com,$date,$ho,$pw,$url,$mlo,$myid,$tim,$up1,$up2,$up3,$col,$tube,$viewkey) = split(/<>/, $par);
	$nam = qq|<a href="mailto:$eml">$nam</a>| if ($eml && $mlo);
	$url &&= qq|<a href="$url" target="_blank">$url</a>|;
	$com =~ s/\{ico:(\d+)\}/<img src="$cf{iconurl}\/$smile[$1]">/g;
	$com =~ s/&amp;/&/g;
	$com =~ s/&#39;/'/g;
	#$com = autolink($com) if ($cf{autolink});

	# 画像
	$com = image($com,$tim,$up1,$up2,$up3,$sub);

	# (BBcode/smailmark) to HTML for SCEditer
	$com = bb_convert($com);

	# YouTube
	if ($cf{tube}) {
		$tube =~ s|!([\w\-]{1,})!|<div class="imageCover"><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="$cf{tube_ico}" width="64" height="64" class="coverImage"></a></div>|g;
	}

	# フォーム用件名
	my $resub = $sub =~ /^Re:/ ? $sub : "Re: $sub";

		# 所定時間以内の投稿は[NEWマーク]表示
		my $time = time;
		my $newsign;
		if ($time - $tim > $cf{new_time} * 3600) { $newsign = ""; }
		else { $newsign = $cf{new_mark}; }

	# 文字置き換え
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!ico!/icon_img($vcue,$res,$alarm,$in{ukey},$tube)/eg;
	$tmpl =~ s/!sub!/$sub/g;
	$tmpl =~ s/!date!/$date $newsign/g;
	$tmpl =~ s/!name!/$nam/g;
	$tmpl =~ s/!url!/$url/g;
	$tmpl =~ s|!comment!|<span style="color:$col[$col]">$com</span>|g;
	$tmpl =~ s|!ico_edit!|<a href="$cf{bbs_cgi}?edit=$in{read}&no=0"><img src="$cf{iconurl}/Edit.png" title="記事編集" alt="記事編集"></a>|g;
	$tmpl =~ s/!color!/$color/g;
	$tmpl =~ s/!page_btn!/$page_btn/g;
	$tmpl =~ s/!res!/$in{read}/g;
	$tmpl =~ s/<!-- youtube -->/$tube/ if ($tube);

	# 認証モード
	if ($cf{authkey}) {
		$tmpl =~ s/!id!/$myid/g;
	} else {
		$tmpl =~ s/<!-- id_begin -->.+?<!-- id_end -->//s;
	}

	# 画像フォーム
	if (!$cf{image_upl}) {
		$tmpl =~ s/<!-- upfile_begin -->.+<!-- upfile_end -->//s;
	}

	# 返信フォーム/編集ボタン
	if (!$resfm) {
		$tmpl =~ s/<!-- resform_begin -->.+<!-- resform_end -->//sg;
		$tmpl =~ s/<!-- edit_begin -->.+?<!-- edit_end -->//sg;
	} else {
		$tmpl =~ s/!str_crypt!/$str_crypt/g;
		$tmpl =~ s/!fm_sub!/$resub/g;
		$tmpl =~ s/!fm_name!/$ck_nam/g;
		$tmpl =~ s/!fm_email!/$ck_eml/g;
		$tmpl =~ s/!fm_url!/$ck_url/g;
		$tmpl =~ s/!smile!/$smile/g;
		$tmpl =~ s/<!-- op_mlo -->/$op_mlo/g;
		$tmpl =~ s/!color!/$color/g;
	}

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3) : error("テンプレート不正");

	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	# レス記事
	foreach (@log) {
		my ($no,$sub,$nam,$eml,$com,$date,$ho,$pw,$url,$mlo,$myid,$tim,$up1,$up2,$up3,$col,$tube,$viewkey) = split(/<>/);
		$nam = qq|<a href="mailto:$eml">$nam</a>| if ($eml && $mlo);
		$url &&= qq|<a href="$url" target="_blank">$url</a>|;
		$com =~ s/\{ico:(\d+)\}/<img src="$cf{iconurl}\/$smile[$1]">/g;
		$com =~ s/&amp;/&/g;
		$com =~ s/&#39;/'/g;
		#$com = autolink($com) if ($cf{autolink});

		# 画像
		$com = image($com,$tim,$up1,$up2,$up3,$sub);

		# (BBcode/smailmark) to HTML for SCEditer
		$com = bb_convert($com);

		# YouTube
		if ($cf{tube}) {
			$tube =~ s|!([\w\-]{1,})!|<div class="imageCover"><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="$cf{tube_ico}" width="64" height="64" class="coverImage"></a></div>|g;
		}

		# 所定時間以内の投稿は[NEWマーク]表示
		my $time = time;
		my $newsign;
		if ($time - $tim > $cf{new_time} * 3600) { $newsign = ""; }
		else { $newsign = $cf{new_mark}; }

		my $tmp = $loop;
		$tmp =~ s|!ico:(\w+\.\w+)!|<img src="$cf{iconurl}/$1" alt="">|g;
		$tmp =~ s/!res_no!/$no/g;
		$tmp =~ s/!res_sub!/$sub/g;
		$tmp =~ s/!res_date!/$date $newsign/g;
		$tmp =~ s/!res_name!/$nam/g;
		$tmp =~ s/!res_url!/$url/g;
		$tmp =~ s|!res_com!|<span style="color:$col[$col]">$com</span>|g;
		$tmp =~ s|!res_ico_edit!|<a href="$cf{bbs_cgi}?edit=$in{read}&no=$no"><img src="$cf{iconurl}/Edit.png" title="記事編集" alt="記事編集"></a>|g;
		$tmp =~ s/<!-- res_youtube -->/$tube/ if ($tube);

		# 認証モード
		if ($cf{authkey}) {
			$tmp =~ s/!res_id!/$myid/g;
		} else {
			$tmp =~ s/<!-- id_begin -->.+?<!-- id_end -->//s;
		}

		print $tmp;
	}

	# カウントアップ
	count_up();

	# フッター
	footer($foot);
}

#-----------------------------------------------------------
#  閲覧用パスワード制限
#-----------------------------------------------------------
sub passwd2 {
	my %in = @_;

	open(READ,"$cf{logdir}/$in{read}.cgi") or &error("open err: $in{read}.cgi");
	my $top = <READ>;
	my $par = <READ>;
	close(READ);
		my ($no2,$sub,$nam,$eml,$com,$date,$ho,$pw,$url,$mlo,$myid,$tim,$up1,$up2,$up3,$col,$tube,$viewkey) = split(/<>/, $par);

		# パスワードがない場合は入力フォーム画面
		if ($in{viewkey} eq "") {
			view_pform();

		# パスワード認証
		} elsif ($in{viewkey} ne $viewkey) {
			error("認証できません");
		}
}

#-----------------------------------------------------------
#  閲覧用パスワード入力フォーム
#-----------------------------------------------------------
sub view_pform {

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/view_pass.html") or error("open err: view_pass.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="$1" class="icon">|g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/style.css|g;
	$tmpl =~ s/!read!/$in{read}/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;

}

#-----------------------------------------------------------
#  BBcodeコンバート (SCEditer対応)
#-----------------------------------------------------------
sub bb_convert {
	$in{read} =~ s/\D//g;

	# ページデータ読み込み
	open(LOG,"$cf{logdir}/$in{read}.cgi") or &error("open err: $in{read}.cgi");
	my $data = join('',<LOG>);
	close(LOG);
		my ($no,$sub,$nam,$eml,$com,$date,$ho,$pw,$url,$mlo,$myid,$tim,$up1,$up2,$up3,$col,$tube) = split(/<>/, $data);

	my $str = $_[0];

	# SCEditer(BBcode) to HTML
	$str =~ s/\[b\](.*?)\[\/b\]/<span style="font-weight:bold">$1<\/span>/ig;
	$str =~ s/\[i\](.*?)\[\/i\]/<span style="font-style:italic">$1<\/span>/ig;
	$str =~ s/\[u\](.*?)\[\/u\]/<span style="text-decoration: underline">$1<\/span>/ig;
	$str =~ s/\[s\](.*?)\[\/s\]/<span style="text-decoration: line-through">$1<\/span>/ig;
	$str =~ s/\[left\](.*?)\[\/left\]/<div id="left">$1<\/div>/ig;
	$str =~ s/\[center\](.*?)\[\/center\]/<div id="center">$1<\/div>/ig;
	$str =~ s/\[right\](.*?)\[\/right\]/<div id="right">$1<\/div>/ig;
	$str =~ s|\[color=([A-Z]+\|\#[0-9A-F]{6})\](.*?)\[\/color\]|<span style="color:\L$1\E">$2</span>|ig;
	$str =~ s/\[size=([\+\-]?[1-7])\](.*?)\[\/size\]/<font size="$1">$2<\/font>/ig;
	$str =~ s|\[youtube\]([\w\-]{1,})(.*?)\[\/youtube\]|<iframe width="854" height="480" src="https://www.youtube.com/embed/$1" frameborder="0" allowfullscreen></iframe>|ig;
	$str =~ s/\[hr(.*?)\]/<hr$1>/ig;
	$str =~ s/\[url=((https?|ftp):\/\/[\w\!\#\$\%\&\'\(\)\*\+\,\-\.\/\:\;\=\?\@\~]*)\](.*?)\[\/url\]/\t<a href="$1" target="_blank" rel="nofollow">$3<\/a>/ig;
	$str =~ s|\[email=(.*?)\](.*?)\[\/email\]|<a href="mailto:$1">$2</a>|ig;
	$str =~ s|\[img\](.*?)\[\/img\]|<a href="$1" class="fancybox-buttons" data-fancybox-group="button" title="$sub"><img src="$1" width="350" height="250" alt="" hspace="15" class="snapshot" /></a>|ig;
	$str =~ s|\[img=(([0-9]{3})(.*?)([0-9]{3})*)\](.*?)\[\/img\]|<a href="$5" class="fancybox-buttons" data-fancybox-group="button" title="$sub"><img src="$5" width="$2" height="$4" alt="" hspace="15" class="snapshot" /></a>|ig;
	$str =~ s/\[ul\]/<ul class="a">/ig;
	$str =~ s/\[li\]/<li>/ig;
	$str =~ s/\[\/ul\]/<\/ul>/ig;
	$str =~ s/\[\/li\]/<\/li>/ig;
	$str =~ s/\[ol\]/<ol class="c">/ig;
	$str =~ s/\[\/ol\]/<\/ol>/ig;
	$str =~ s/\[list\](.*?)\[\/list\]/<ul>$1<\/ul>/ig;
	$str =~ s/\[list=(1|A)\](.*?)\[\/list\]/<ol type="$1">$2<\/ol>/ig;
	$str =~ s/\[\*\]/<li>/ig;
	$str =~ s/\[code\](.*?)\[\/code\]/<code>$1<\/code>/ig;
	$str =~ s|\[quote\](.*?)\[\/quote\]|<blockquote><div id="quote_ttl">[引用]</div>$1<\/blockquote>|ig;

	# SCEditer(smailmark) to HTML
	$str =~ s|:alien:|<img src="$cf{emoticonurl}\/alien.png" alt="" />|g;
	$str =~ s|:blink:|<img src="$cf{emoticonurl}\/blink.png" alt="" />|g;
	$str =~ s|:blush:|<img src="$cf{emoticonurl}\/blush.png" alt="" />|g;
	$str =~ s|:cheerful:|<img src="$cf{emoticonurl}\/cheerful.png" alt="" />|g;
	$str =~ s|:devil:|<img src="$cf{emoticonurl}\/devil.png" alt="" />|g;
	$str =~ s|:dizzy:|<img src="$cf{emoticonurl}\/dizzy.png" alt="" />|g;
	$str =~ s|:getlost:|<img src="$cf{emoticonurl}\/getlost.png" alt="" />|g;
	$str =~ s|:happy:|<img src="$cf{emoticonurl}\/happy.png" alt="" />|g;
	$str =~ s|:kissing:|<img src="$cf{emoticonurl}\/kissing.png" alt="" />|g;
	$str =~ s|:ninja:|<img src="$cf{emoticonurl}\/ninja.png" alt="" />|g;
	$str =~ s|:pinch:|<img src="$cf{emoticonurl}\/pinch.png" alt="" />|g;
	$str =~ s|:pouty:|<img src="$cf{emoticonurl}\/pouty.png" alt="" />|g;
	$str =~ s|:sick:|<img src="$cf{emoticonurl}\/sick.png" alt="" />|g;
	$str =~ s|:sideways:|<img src="$cf{emoticonurl}\/sideways.png" alt="" />|g;
	$str =~ s|:silly:|<img src="$cf{emoticonurl}\/silly.png" alt="" />|g;
	$str =~ s|:sleeping:|<img src="$cf{emoticonurl}\/sleeping.png" alt="" />|g;
	$str =~ s|:unsure:|<img src="$cf{emoticonurl}\/unsure.png" alt="" />|g;
	$str =~ s|:woot:|<img src="$cf{emoticonurl}\/w00t.png" alt="" />|g;
	$str =~ s|:wassat:|<img src="$cf{emoticonurl}\/wassat.png" alt="" />|g;
	$str =~ s|:whistling:|<img src="$cf{emoticonurl}\/whistling.png" alt="" />|g;
	$str =~ s|:love:|<img src="$cf{emoticonurl}\/wub.png" alt="" />|g;
	$str =~ s|:smile:|<img src="$cf{emoticonurl}\/smile.png" alt="" />|g;
	$str =~ s|:angel:|<img src="$cf{emoticonurl}\/angel.png" alt="" />|g;
	$str =~ s|:angry:|<img src="$cf{emoticonurl}\/angry.png" alt="" />|g;
	$str =~ s|:cool:|<img src="$cf{emoticonurl}\/cool.png" alt="" />|g;
	$str =~ s|:cwy:|<img src="$cf{emoticonurl}\/cwy.png" alt="" />|g;
	$str =~ s|:ermm:|<img src="$cf{emoticonurl}\/ermm.png" alt="" />|g;
	$str =~ s|:D|<img src="$cf{emoticonurl}\/grin.png" alt="" />|g;
	$str =~ s|:heart:|<img src="$cf{emoticonurl}\/heart.png" alt="" />|g;
	$str =~ s|:sad:|<img src="$cf{emoticonurl}\/sad.png" alt="" />|g;
	$str =~ s|:wink:|<img src="$cf{emoticonurl}\/wink.png" alt="" />|g;
	$str =~ s|:O|<img src="$cf{emoticonurl}\/shocked.png" alt="" />|g;
	$str =~ s|:P|<img src="$cf{emoticonurl}\/tongue.png" alt="" />|g;

	return $str;
}

#-----------------------------------------------------------
#  カウンタ処理
#-----------------------------------------------------------
sub bbs_count {
	# IP取得
	my $addr = $ENV{REMOTE_ADDR};

	# 閲覧時のみカウントアップ
	my $cntup = $in{mode} eq '' ? 1 : 0;

	# カウントファイルを読みこみ
	open(LOG,"+< $cf{cntfile}") or error("open err: $cf{cntfile}");
	eval "flock(LOG, 2);";
	my $count = <LOG>;

	# IPチェックとログ破損チェック
	my ($cnt,$ip) = split(/:/,$count);
	if ($addr eq $ip or $cnt eq "") { $cntup = 0; }

	# カウントアップ
	if ($cntup) {
		$cnt++;
		seek(LOG, 0, 0);
		print LOG "$cnt:$addr";
		truncate(LOG, tell(LOG));
	}
	close(LOG);

	# 桁数調整
	while(length($cnt) < $cf{mini_fig}) { $cnt = '0' . $cnt; }
	my @cnts = split(//,$cnt);

	# GIFカウンタ表示
	my $counter;
	if ($cf{counter} == 2) {
		foreach (0 .. $#cnts) {
			$counter .= qq|<img src="$cf{gif_path}/$cnts[$_].gif\" alt="$cnts[$_]" />|;
		}

	# テキストカウンタ表示
	} else {
		$counter = qq|<span style="color:$cf{cntcol}">$cnt</span>\n|;
	}
	return $counter;
}

#-----------------------------------------------------------
#  YouTube貼付方法
#-----------------------------------------------------------
sub expound_page {

	open(IN,"$cf{tmpldir}/expound.html  ") or error("open err: expound.html  ");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  アルバム機能
#-----------------------------------------------------------
sub album_page {

	# ページ数定義
	my $pg = $in{pg} || 0;

	# 画像サイズ再定義
	$cf{max_img_w} = $cf{alb_img_w};
	$cf{max_img_h} = $cf{alb_img_h};

	# テンプレート認識
	open(IN,"$cf{tmpldir}/album.html") or error("open err: album.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl);

	# indexファイル
	my ($i,@img);
	open(NUM,"$cf{nowfile}") or error("open err: $cf{nowfile}");
	my $top = <NUM>;
	while(<NUM>) {
		my ($num) = (split(/<>/))[0];

		# データ読み込み
		open(DAT,"< $cf{logdir}/$num.cgi") or &error("open err: $cf{logdir}/$num.cgi");
		my $top = <DAT>;
		while(<DAT>) {
			my ($no,$sub,$nam,$eml,$com,$dat,$ho,$pw,$url,$mlo,$myid,$tim,$up1,$up2,$up3,$col) = split(/<>/);

	#my %img = (1 => $up1, 2 => $up2, 3 => $up3);
	#my %ex = (1 => $ex1, 2 => $ex2, 3 => $ex3);
	#foreach my $i (1 .. 3) {
	#	my $ex{$i} = split(/,/, $img{$i});
	#		next if ($ex{$i} !~ /(\.jpe?g|\.png|\.gif)$/i);
	#}
			my $ex1 = (split(/,/, $up1))[0];
			next if ($ex1 !~ /(jpg|png|gif)$/);

			$i++;
			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{max_albums});

			# 画像データ収集
			my $ex1 = (split(/,/, $up1))[0];
			my $ex2 = (split(/,/, $up2))[0];
			my $ex3 = (split(/,/, $up3))[0];

			if ($ex1) { push(@img,"$sub\t$nam\t$tim\t$up1");}
			if ($ex2) { push(@img,"$sub\t$nam\t$tim\t$up1\t$up2");}
			if ($ex3) { push(@img,"$sub\t$nam\t$tim\t$up1\t$up2\t$up3");}
		}
		close(DAT);
	}
	close(NUM);

	# 繰越ボタン
	my $page_btn = album_pgbtn($i,$pg,'&mode=album',$cf{max_albums});

	# 文字置換
	for ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!page_btn!/$page_btn/g;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@img) {
		my ($sub,$nam,$tim,$up1,$up2,$up3) = split(/\t/);

		my $tmp = $loop;
		$tmp =~ s/!image!/image2($sub,$tim,$up1,$up2,$up3)/eg;
		$tmp =~ s/!caption!/$sub/g;
		$tmp =~ s/!name!/$nam/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  画像表示 (album)
#-----------------------------------------------------------
sub image2 {
	my ($sub,$fnam,$up1,$up2,$up3) = @_;
	my %img = (1 => $up1, 2 => $up2, 3 => $up3);

	my $img;
	foreach my $i (1 .. 3) {
		my ($ex,$w,$h) = split(/,/, $img{$i});
		if ($ex) {
			($w,$h) = &resize($w,$h);
			my $pic = -e "$cf{uplurl}/$fnam-s-$i$ex" ? "$fnam-s-$i$ex" : "$fnam-$i$ex";
			$img = qq|<a class="fancybox-buttons" data-fancybox-group="button" href="$cf{uplurl}/$fnam-$i$ex" target="_blank" title="$sub"><img src="$cf{uplurl}/$pic" width="$w" height="$h" hspace="2"></a>|;
		}
	}
	if ($img) {
		return "<p>$img</p>";
	}
}

#-----------------------------------------------------------
#  繰越ボタン作成 (アルバム)
#-----------------------------------------------------------
sub album_pgbtn {
	my ($i,$pg,$stat,$max) = @_;
	$max ||= 10;

	# ページ繰越定義
	my $next = $pg + $max;
	my $back = $pg - $max;

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?pg=$y$stat">$x</a></li>!);
			}
			$x++;
			$y += $max;
			$z -= $max;

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?pg=$back$stat">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?pg=$next$stat">&raquo;</a></li>\n!;
	}
	return $ret ne '' ? qq|<ul class="pager">$ret</ul>| : '';
}

#-----------------------------------------------------------
#  認証フォーム
#-----------------------------------------------------------
sub pwd_form {
	# 汚染チェック
	$in{edit} =~ s/\D//g;
	$in{no} =~ s/\D//g;

	# 記事
	my ($name,$pwd);
	open(IN,"$cf{logdir}/$in{edit}.cgi") or &error("open err: $in{edit}.cgi");
	my $top = <IN>;
	while(<IN>) {
		my ($no,$sub,$nam,$eml,$com,$dat,$ho,$pw,$url,$mlo,$myid,$tim,$upl1,$upl2,$upl3) = split(/<>/);

		if ($in{no} == $no) {
			$name = $nam;
			$pwd = $pw;
			last;
		}
	}
	close(IN);

	# パスワード未設定
	error("この記事はパスワード未設定のため編集できません") if ($pwd eq '');

	# 処理選択オプション
	my $op_job = qq|<option value="edit" selected>記事を編集\n|;
	if ($in{no} == 0) {
		$op_job .= qq|<option value="lock">スレッドをロック\n|;
	} else {
		$op_job .= qq|<option value="dele">記事を削除\n|;
	}

	# 記事情報データ分解
	my ($no,$sub,$res,$key) = split(/<>/,$top);

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/pwd.html") or &error("open err: pwd.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!sub!/$sub/g;
	$tmpl =~ s/!name!/$name/g;
	$tmpl =~ s/!art!/$in{edit}/g;
	$tmpl =~ s/!no!/$in{no}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{iconurl}/$1">|g;
	$tmpl =~ s/<!-- op_job -->/$op_job/g;

	# 画面表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  留意事項表示
#-----------------------------------------------------------
sub note_page {
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/note.html") or &error("open err: note.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!maxdata!/$cf{maxdata}バイト/g;
	$tmpl =~ s/!max_w!/$cf{max_img_w}/g;
	$tmpl =~ s/!max_h!/$cf{max_img_h}/g;

	# 画面表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  画像表示
#-----------------------------------------------------------
sub image {
	my ($com,$fnam,$up1,$up2,$up3,$sub) = @_;
	my %img = (1 => $up1, 2 => $up2, 3 => $up3);

	my $img;
	foreach my $i (1 .. 3) {
		my ($ex,$w,$h) = split(/,/, $img{$i});
		# 指定アイコン
		if (defined($cf{icons}{$ex})) {
			my $size = get_size("$cf{uplurl}/$fnam-$i$ex");
			$img .= qq|<a href="$cf{uplurl}/$fnam-$i$ex" target="_blank"><img src="$cf{iconurl}/$cf{icons}{$ex}" alt="" class="icon" /></a> ($size)\n|;

		# 画像の場合
		} elsif ($ex) {
			($w,$h) = &resize($w,$h);
			my $pic = -e "$cf{uplurl}/$fnam-s-$i$ex" ? "$fnam-s-$i$ex" : "$fnam-$i$ex";
			$img .= qq|<a class="fancybox-effects-c" href="$cf{uplurl}/$fnam-$i$ex" title="タイトル：$sub"><img src="$cf{uplurl}/$pic" width="$w" height="$h" hspace="2" class="snapshot"></a>|;
		# 画像以外
		} elsif ($ex =~ /^\.(jpg|gif|png)$/) {
			my $size = get_size("$cf{uplurl}/$fnam-$i$ex");
			$img .= qq|[添付]: <a href="$cf{uplurl}/$fnam-$i$ex" target="_blank">$cf{uplurl}/$fnam-$i$ex</a> ($size)\n|;
		}
	}
	if ($img) {
		return "$com<p>$img</p>";
	} else {
		return $com;
	}
}

#-----------------------------------------------------------
#  ファイル容量
#-----------------------------------------------------------
sub get_size {
	my $file = shift;
	
	my $size = -s $file || 0;
	return $size > 1024000 ? comma(int($size/1024000)) . 'MB' : $size > 1024 ? comma(int($size/1024)) . 'KB' : $size . 'Bytes';
}

#-----------------------------------------------------------
#  桁区切り
#-----------------------------------------------------------
sub comma {
	local($_) = @_;

	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
}

#-----------------------------------------------------------
#  投稿フォーム
#-----------------------------------------------------------
sub form_page {
	# クッキー取得
	my ($ck_nam,$ck_eml,$ck_url,$ck_mlo,$ck_col) = &get_cookie;
	$ck_url ||= 'http://';

	# 色選択ボタン
	my @col = split(/\s+/, $cf{colors});
	my $color;
	foreach (0 .. $#col) {
		if ($ck_col == $_) {
			$color .= qq|<input type="radio" name="color" value="$_" checked>|;
		} else {
			$color .= qq|<input type="radio" name="color" value="$_">|;
		}
		$color .= qq|<span style="color:$col[$_]">●</span>\n|;
	}

	# アイコン
	my $smile;
	my @smile = split(/\s+/, $cf{smile});
	foreach (0 .. $#smile) {
		$smile .= qq|<a href="javascript:face('{ico:$_}')"><img src="$cf{iconurl}/$smile[$_]"></a>|;
	}

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/form.html") or &error("open err: form.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# email表示
	my $op_mlo;
	my @mlo = ('非表示','表示');
	foreach (1,0) {
		if ($ck_mlo eq $_) {
			$op_mlo .= qq|<option value="$_" selected>$mlo[$_]\n|;
		} else {
			$op_mlo .= qq|<option value="$_">$mlo[$_]\n|;
		}
	}

	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt) = cap::make( $cf{captcha_key}, $cf{cap_len} );
	} else {
		$tmpl =~ s/<!-- captcha_begin -->.+<!-- captcha_end -->//s;
	}

	# 画像フォーム
	if (!$cf{image_upl}) {
		$tmpl =~ s/<!-- upfile_begin -->.+<!-- upfile_end -->//s;
	}

	# 文字置換
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!ico:(\w+\.\w+)!/<img src="$cf{iconurl}\/$1">/g;
	$tmpl =~ s/!str_crypt!/$str_crypt/g;
	$tmpl =~ s/!name!/$ck_nam/g;
	$tmpl =~ s/!email!/$ck_eml/g;
	$tmpl =~ s/!url!/$ck_url/g;
	$tmpl =~ s/!smile!/$smile/g;
	$tmpl =~ s/<!-- op_mlo -->/$op_mlo/g;
	$tmpl =~ s/!color!/$color/g;

	# 画面表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  TOP検索
#-----------------------------------------------------------
sub find_log {
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/find.html") or &error("open err: find.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 条件
	if ($in{cond} eq '') { $in{cond} = 1; }
	my $op_cond;
	my %cond = (1 => 'AND', 0 => 'OR');
	foreach (1,0) {
		if ($in{cond} == $_) {
			$op_cond .= qq|<option value="$_" selected>$cond{$_}\n|;
		} else {
			$op_cond .= qq|<option value="$_">$cond{$_}\n|;
		}
	}
	# 範囲
	$in{area} ||= 1;
	my %area = (1 => '現行ログ', 2 => '過去ログ');
	my $area;
	foreach (1,2) {
		if ($in{area} == $_) {
			$area .= qq|<input type="radio" name="area" value="$_" checked>$area{$_}\n|;
		} else {
			$area .= qq|<input type="radio" name="area" value="$_">$area{$_}\n|;
		}
	}

	# 文字置換
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!ico:(\w+\.\w+)!/<img src="$cf{iconurl}\/$1">/g;
	$tmpl =~ s/<!-- op_cond -->/$op_cond/g;
	$tmpl =~ s/!area!/$area/g;
	$tmpl =~ s/!word!/$in{word}/g;

	# 検索実行
	if ($in{word} ne '') {
		my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- search -->(.+?)<!-- /search -->(.+)|s
				? ($1,$2,$3)
				: error("テンプレートが不正です$tmpl");
		
		# 画面表示
		print "Content-type: text/html; charset=utf-8\n\n";
		print $head;
		search($loop);
		print $foot;
	# フッター
	footer();
		exit;

	# 検索前
	} else {
		$tmpl =~ s|<!-- search -->.+?<!-- /search -->||s;
		
		# 画面表示
		print "Content-type: text/html; charset=utf-8\n\n";
		print $tmpl;

	# フッター
	footer();
		exit;
	}
}

#-----------------------------------------------------------
#  検索画面
#-----------------------------------------------------------
sub find_page {
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/find.html") or &error("open err: find.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 条件
	if ($in{cond} eq '') { $in{cond} = 1; }
	my $op_cond;
	my %cond = (1 => 'AND', 0 => 'OR');
	foreach (1,0) {
		if ($in{cond} == $_) {
			$op_cond .= qq|<option value="$_" selected>$cond{$_}\n|;
		} else {
			$op_cond .= qq|<option value="$_">$cond{$_}\n|;
		}
	}
	# 範囲
	$in{area} ||= 1;
	my %area = (1 => '現行ログ', 2 => '過去ログ');
	my $area;
	foreach (1,2) {
		if ($in{area} == $_) {
			$area .= qq|<input type="radio" name="area" value="$_" checked>$area{$_}\n|;
		} else {
			$area .= qq|<input type="radio" name="area" value="$_">$area{$_}\n|;
		}
	}

	# 文字置換
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!ico:(\w+\.\w+)!/<img src="$cf{iconurl}\/$1">/g;
	$tmpl =~ s/<!-- op_cond -->/$op_cond/g;
	$tmpl =~ s/!area!/$area/g;
	$tmpl =~ s/!word!/$in{word}/g;

	# 検索実行
	if ($in{word} ne '') {
		my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- search -->(.+?)<!-- /search -->(.+)|s
				? ($1,$2,$3)
				: error("テンプレートが不正です$tmpl");
		
		# 画面表示
		print "Content-type: text/html; charset=utf-8\n\n";
		print $head;
		search($loop);
		print $foot;
	# フッター
	footer();
		exit;

	# 検索前
	} else {
		$tmpl =~ s|<!-- search -->.+?<!-- /search -->||s;
		
		# 画面表示
		print "Content-type: text/html; charset=utf-8\n\n";
		print $tmpl;

	# フッター
	footer();
		exit;
	}
}

#-----------------------------------------------------------
#  検索実行
#-----------------------------------------------------------
sub search {
	my $loop = shift;
	$loop =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s;
	my ($head,$loop,$foot) = ($1,$2,$3);
	
	$in{word} =~ s/　/ /g;
	my @wd = split(/\s+/,$in{word});
	
	# 対象範囲/引数
	my ($idx,$param);
	if ($in{area} == 1) {
		$idx = $cf{nowfile};
	} else {
		$idx = $cf{pastfile};
		$param = "&amp;log=past";
	}
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

	# indexオープン
	my @find;
	open(IN,"$idx") or error("open err: $idx");
	my $top = <IN>;
	while(<IN>) {
		my ($num,$sub,$res,$nam,$upd,$last,$key,$upl) = split(/<>/);

		# 各スレッド読み込み
		open(LOG,"$cf{logdir}/$num.cgi");
		my $log = join('', <LOG>);
		close(LOG);

		# スレッド内検索
		my $flg;
		foreach my $wd (@wd) {
			if ($log =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($in{cond} == 0) { last; }
			} else {
				if ($in{cond} == 1) { $flg = 0; last; }
			}
		}
		if ($flg) { push(@find,$_); }
	}
	close(IN);

	# ヒット件数
	my $hit = @find;
	$head =~ s/!hit!/$hit/g;
	$foot =~ s/!hit!/$hit/g;

	# アラーム数定義
	my $alarm = int ( $cf{m_max} * 0.9 );

	# 結果表示
	print $head;
	foreach (@find) {
		my ($num,$sub,$res,$nam,$upd,$last,$key,$upl,$tube,$readkey) = split(/<>/);
		my $ukey = $upl ? 1 : 0;

		# データ読み込み
		open(DAT,"< $cf{logdir}/$num.cgi") or &error("open err: $cf{logdir}/$num.cgi");
		my $top = <DAT>;
		my $par = <DAT>;
			my ($no2,$tube) = (split(/<>/, $par))[0,16];

		my $tmp = $loop;
		$tmp =~ s/!icon!/icon_img($key,$res,$alarm,$upl,$tube,$readkey)/eg;
		$tmp =~ s|!sub!|<a href="$cf{bbs_cgi}?read=$num&ukey=$ukey$param">$sub</a>|g;
		$tmp =~ s/!name!/$nam/g;
		$tmp =~ s/!res!/$res/g;
		$tmp =~ s/!update!/$upd/g;
		print $tmp;
	}
	print $foot;
}

#-----------------------------------------------------------
#  過去ログページ
#-----------------------------------------------------------
sub past_page {
	# ページ数
	my $pg = $in{pg} || 0;

	# スレッド表示
	my ($i,@log);
	open(IN,"$cf{pastfile}") or error("open err: $cf{pastfile}");
	while (<IN>) {
		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{pgmax_past});

		push(@log,$_);
	}
	close(IN);

	# 繰越ボタン作成
	my $page_btn = make_pgbtn($i,$pg,$cf{pgmax_past},"mode=past");

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/past.html") or error("open err: past.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!ico:(\w+\.\w+)!/<img src="$cf{iconurl}\/$1">/g;
	$tmpl =~ s/!page-btn!/$page_btn/g;

	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3) : error("テンプレート不正");

	# 画面表示
		print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@log) {
		my ($num,$sub,$res,$nam,$upd,$last,$key,$upl) = split(/<>/);

		my $tmp = $loop;
		$tmp =~ s|!icon!|<img src="$cf{iconurl}/fld_nor.gif">|;
		$tmp =~ s|!sub!|<a href="$cf{bbs_cgi}?read=$num&log=past">$sub</a>|g;
		$tmp =~ s/!name!/$nam/g;
		$tmp =~ s/!res!/$res/g;
		$tmp =~ s/!update!/$upd/g;
		print $tmp;
	}

	# フッター
	footer($foot);
}

#-----------------------------------------------------------
#  繰越ボタン作成
#-----------------------------------------------------------
sub make_pgbtn {
	my ($i,$pg,$max,$param) = @_;
	$max ||= 10;

	# 引数
	$param &&= "&$param";

	# ページ繰越定義
	my $next = $pg + $max;
	my $back = $pg - $max;

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?pg=$y$param">$x</a></li>!);
			}
			$x++;
			$y += $max;
			$z -= $max;

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?pg=$back$param">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?pg=$next$param">&raquo;</a></li>\n!;
	}
	return $ret ne '' ? qq|<ul class="pager">$ret</ul>| : '';
}

#-----------------------------------------------------------
#  カウントアップ
#-----------------------------------------------------------
sub count_up {
	# IP取得
	my $addr = $ENV{REMOTE_ADDR};

	# カウントデータオープン
	open(DAT,"+< $cf{logdir}/$in{read}.dat") or error("open err: $in{read}.dat");
	eval "flock(DAT, 2);";
	my $data = <DAT>;
	my ($cnt,$ip) = split(/:/,$data);

	# IPチェック
	if ($addr ne $ip) {
		$cnt++;
		seek(DAT, 0, 0);
		print DAT "$cnt:$addr";
		truncate(DAT, tell(DAT));
	}
	close(DAT);
}

#-----------------------------------------------------------
#  認証モード
#-----------------------------------------------------------
sub authent {
	# セッションモジュール取り込み
	require $cf{session_pl};

	# ログイン
	if ($in{mode} eq 'login') {
		# 入力チェック
		if ($in{id} eq '' || $in{pw} eq '') {
			error("IDまたはパスワードが未入力です");
		}
		# セッション作成
		make_ses($in{id},$in{pw},$cf{memfile},$cf{sesdir},$cf{authtime});

	# ログオフ
	} elsif ($in{mode} eq 'logoff') {
		# セッション削除
		del_ses($cf{sesdir});

		# 入室画面
		enter_form('cook_del');

	# 入室画面
	} elsif ($in{mode} eq 'enter') {
		enter_form();

	# セッション管理
	} else {
		session($cf{sesdir},$cf{bbs_cgi});
	}
}

#-----------------------------------------------------------
#  入室画面
#-----------------------------------------------------------
sub enter_form {
	my $ck = shift;

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/enter.html") or error("open err: enter.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/patio.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/patio-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;

	# クッキー排除
	if ($ck eq 'cook_del') { set_cookie('CGISESSID','','del'); }

	# 画面表示
		print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  crypt照合
#-----------------------------------------------------------
sub decrypt {
	my ($in, $dec) = @_;

	my $salt = $dec =~ /^\$1\$(.*)\$/ ? $1 : substr($dec, 0, 2);
	if (crypt($in, $salt) eq $dec || crypt($in, '$1$' . $salt) eq $dec) {
		return 1;
	} else {
		return 0;
	}
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキー取得
	my $cook = $ENV{HTTP_COOKIE};

	# 該当IDを取り出す
	my %cook;
	foreach ( split(/;/, $cook) ) {
		my ($key,$val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}

	# URLデコード
	my @cook;
	foreach ( split(/<>/, $cook{$cf{cookie_id}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1)/eg;
		s/[&"'<>]//g;

		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my ($key,$val,$del) = @_;

	# 時間定義
	my $gtime = $del eq 'del' ? time - 365*24*60*60 : time + 60*24*60*60;

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,undef,undef) = gmtime($gtime);
	my @mon  = qw|Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec|;
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;

	# 時刻フォーマット
	my $gmt = sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",
				$week[$wday],$mday,$mon[$mon],$year+1900,$hour,$min,$sec);

	print "Set-Cookie: $key=$val; expires=$gmt\n";
}

#-----------------------------------------------------------
#  自動リンク
#-----------------------------------------------------------
sub autolink {
	my $text = shift;

	$text =~ s/(s?https?:\/\/([\w-.!~*'();\/?:\@=+\$,%#]|&amp;)+)/<a href="$1" target="_blank">$1<\/a>/g;
	return $text;
}

